<?php
/**
 * Fastway Plugin for Hikashop is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Busy Bee School Administration Software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Busy Bee.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package		hikashopshippingfastway

 * @version	29th February 2016
 * @since 	24th April 2015
 */

defined('_JEXEC') or die();


if (! function_exists('dump')) Jloader::import('plugins.hikashopshipping.aufastway.debug', JPATH_ROOT);
/**
 * Fastway eDelivery Class for HikaShop
 *
 * @version 29th February 2016
 * @since	24th April 2015
 */
class plgHikashopshippingAufastway extends hikashopShippingPlugin {

	var $baseURL = "http://au.api.fastway.org/v3/psc/";
	var $fastway = NULL;
	var $multiple = true;
	var $name = 'aufastway';
	var $doc_form = 'aufastway';
	var $use_cache = false;
	var $pluginConfig = array(
		'api_code' => array ( 'Fastway API Code', 'input' ) ,
		'post_code' => array( 'Pickup PostCode', 'input'),
		'pickup_locality' 	=> array( 'Pick-Up Locality', 'input'),
		'services' 			=> array('SHIPPING_SERVICES', 'checkbox', array(
			'PARCEL' 			=> 'Parcel',
			'SATCHEL' 			=> 'Satchel',
		)),
		'frequent_pickup' 	=> array('Use Frequent Prices', 'boolean', '0' ),
		'reverse_order'		=> array('Reverse order of services', 'boolean', '0'),
		'shipping_group' 	=> array('Group products together', 'boolean', '0'),
		'limit_price' 		=> array('If Price setting (above) is greater than zero, is this a maxiumum price to charge, or a fixed price.  Fixed (Yes), Maximum (No)', 'boolean', '1'),
		'free_shipping' 	=> array('Free Shipping', 'boolean', '0'),
		'suppress_errors' 	=> array('Suppress Error Messages', 'boolean', '0'),
		'lowest_only' 		=> array('Show Lowest Price Only', 'boolean', 1),
	);
	var $methods = array(
		'PARCEL' => 1,
		'SATCHEL' => 2
	);
/**
 * Fastway Process Packing Limit
 *
 * @version 	4th August 2015
 * @since	 	24th April 2015
 */
	function processPackageLimit($limit_key, $limit_value, $product, $qty, $package, $units) {

		switch ($limit_key) {
			case 'volume':
				if(($product['x'] * $product['y'] * $product['z']) > $limit_value)
					return false;
				if(($product['x'] * $product['y'] * $product['z']) == 0) {
					$this->queueError(JText::_('DIMENSIONS_ERROR'));
					return false;
				}
				return (int)floor($limit_value / ($product['x'] * $product['y'] * $product['z']));
				break;
			case 'girth':
				if(($product['x'] + $product['y']) * 2  > $limit_value)
					return false;
				return (int)floor($limit_value / (($product['x'] + $product['y']) * 2 ));
				break;
		}
		return parent::processPackageLimit($limit_key, $limit_value , $product, $qty, $package, $units);
	}
/**
 * Fastway On Shipping Display
 *
 * @version 	24th April 2015
 * @since	 	24th April 2015
 */
	function onShippingDisplay(&$order,&$dbrates,&$usable_rates,&$messages){
		if(!hikashop_loadUser())
			return false;

		if($this->loadShippingCache($order, $usable_rates, $messages))
			return true;

		$local_usable_rates = array();
		$local_messages = array();
		$currencyClass = hikashop_get('class.currency');
		$ret = parent::onShippingDisplay($order, $dbrates, $local_usable_rates, $local_messages);
		if($ret === false)
			return false;

		$cache_usable_rates = array();
		$cache_messages = array();

		$currentCurrencyId = null;
		foreach($local_usable_rates as $rate) {
			if(!empty($rate->shipping_zone_namekey)){
				if(empty($rate->shipping_params->SEA) && empty($rate->shipping_params->AIR) && !empty($order->shipping_address->address_country)){

					$db = JFactory::getDBO();
					if(is_array($order->shipping_address->address_country)){
						$address_country = reset($order->shipping_address->address_country);
					}else{
						$address_country = $order->shipping_address->address_country;
					}
					$db->setQuery('SELECT * FROM '.hikashop_table('zone').' WHERE zone_namekey='.$db->Quote($address_country));
					$currentShippingZone = $db->loadObject();

					if($currentShippingZone->zone_code_3 != 'AUS') {
						$messages['no_shipping_to_your_zone'] = JText::_('NO_SHIPPING_TO_YOUR_ZONE');
						continue;
					}
				}
			}

			$check = false;

			if(empty($order->shipping_address->address_post_code)){
				$check = true;
				$message = 'The Fastway shipping plugin requires the user to enter a postal code when goods are shipped within Australia. Please go to "Display->Custom fields" and set the post code field to required.';
			}elseif(!preg_match('#[0-9]{4}#',$order->shipping_address->address_post_code)){
				$check = true;
				$message = 'The post code entered is not valid';
				$order->shipping_address->address_post_code = preg_replace('#[^0-9A-Z]#','',$order->shipping_address->address_post_code);
			}
			if($check){
				$zoneClass=hikashop_get('class.zone');
				$zones = $zoneClass->getOrderZones($order);
				$db = JFactory::getDBO();
				$db->setQuery('SELECT zone_namekey FROM '.hikashop_table('zone').' WHERE zone_code_3='.$db->Quote('AUS'));
				$australia_zone = $db->loadResult();
				if(in_array($australia_zone,$zones)){
					$cache_messages['post_code_missing'] = $message;
					continue;
				}
			}
			if(empty($order->shipping_address_full)){
				$cart = hikashop_get('class.cart');
				$app = JFactory::getApplication();
				$address=$app->getUserState( HIKASHOP_COMPONENT.'.shipping_address');
				$cart->loadAddress($order->shipping_address_full,$address,'object','shipping');
			}
			$rates = array();

			$this->getRates($rate, $order, $rates);

			$w = reset($rates);

			if ($w->shipping_params->lowest_only){
				$cheapest = '';
				foreach($rates as $q=>$w) {
					if (empty($cheapest))
						$cheapest = $q;
					if ($w->shipping_price < $rates[$cheapest]->shipping_price)
						$cheapest = $q;
				}
				$w = $rates[$cheapest];
				$rates = array($cheapest => $w);
			}

			if(!empty($rate->shipping_params->reverse_order)){
				$rates=array_reverse($rates,true);
			}
			foreach($rates as $finalRate){
				$usable_rates[$finalRate->shipping_id] = $finalRate;
				$cache_usable_rates[$finalRate->shipping_id] = $finalRate;
			}
		}
		$this->setShippingCache($order, $cache_usable_rates, $cache_messages);

		if(!empty($cache_messages)) {
			foreach($cache_messages as $k => $msg) {
				$messages[$k] = $msg;
			}
		}
		return true;
	}
/**
 * Fastway Get Rates
 *
 * @version 	27th April 2015
 * @since	 	24th April 2015
 */
	function getRates($rate, $order, &$rates){

		$lang = JFactory::getLanguage()->load('plg_hikashopshipping_aufastway', JPATH_ADMINISTRATOR);
		$this->fastway = $this->resetFastwayObject();
		$this->fastway->address_city = $order->shipping_address->address_city;
		$weightClass=hikashop_get('helper.weight');
		$volumeClass=hikashop_get('helper.volume');
		$limit = array();
		if(@$order->shipping_address_full->shipping_address->address_country->zone_code_2 == 'AU'){
			$limit['w'] = 25000;
			$limit['volume'] = 250000000;
			$limit['x'] = 1050;
		}else{
			$limit['w'] = 20000;
			$limit['girth'] = 1400;
			$limit['x'] = 1050;
		}
		if(empty($currentCurrencyId)) {
			$query = "SELECT `currency_id` FROM `".hikashop_table('currency')."` WHERE currency_code='AUD'";
			$db = JFactory::getDBO();
			$db->setQuery($query);
			$currentCurrencyId = $db->loadResult();
		}
		$parcel=new stdClass();
		$parcel->Weight = 0;
		$parcel->global_volume = 0;
		$parcel->Width = 0;
		$parcel->Height = 0;
		$parcel->Length = 0;
		$parcels = array($parcel);
		$i=0;

		if(isset($rate->shipping_params->shipping_group) && $rate->shipping_params->shipping_group){
			$packages = $this->getOrderPackage($order, array('weight_unit' => 'g', 'volume_unit' => 'mm', 'limit' => $limit));

			if(empty($packages))
				return true;
			if(isset($packages['w']) || isset($packages['x']) || isset($packages['y']) || isset($packages['z'])){
				if(empty($parcels[$i]))
					$parcels[$i] = new stdClass();
				$parcels[$i]->Weight = $packages['w'];
				$parcels[$i]->Width = $packages['z'];
				$parcels[$i]->Height = $packages['y'];
				$parcels[$i]->Length = $packages['x'];

				if($parcels[$i]->Weight<1)$parcels[$i]->Weight=1;
				if($parcels[$i]->Length<150)$parcels[$i]->Length=150;
				if($parcels[$i]->Width<150)$parcels[$i]->Width=150;
				if($parcels[$i]->Height<1)$parcels[$i]->Height=1;

				$i++;
			}else{
				foreach($packages as $package){
					if(empty($parcels[$i]))
						$parcels[$i] = new stdClass();
					$parcels[$i]->Weight = $package['w'];
					$parcels[$i]->Width = $package['z'];
					$parcels[$i]->Height = $package['y'];
					$parcels[$i]->Length = $package['x'];

					if($parcels[$i]->Weight<1)$parcels[$i]->Weight=1;
					if($parcels[$i]->Length<150)$parcels[$i]->Length=150;
					if($parcels[$i]->Width<150)$parcels[$i]->Width=150;
					if($parcels[$i]->Height<1)$parcels[$i]->Height=1;

					$i++;
				}
			}
			foreach($parcels as $parcel){
				$parcel->Country = @$order->shipping_address_full->shipping_address->address_country->zone_code_2;
				if(empty($parcel->Country)) $parcel->Country='AU';
				$parcel->Pickup_Postcode = substr(trim(@$rate->shipping_params->post_code),0,4);
				$parcel->Destination_Postcode = substr(trim($order->shipping_address->address_post_code),0,4);
				$parcel->Quantity = 1;
				if(!empty($rate->shipping_params->PARCEL)){
					$this->addRate($rates,'PARCEL',$parcel,$rate,$currentCurrencyId, $i);
				}
				if(!empty($rate->shipping_params->SATCHEL)){
					$this->addRate($rates,'SATCHEL',$parcel,$rate,$currentCurrencyId, $i);
				}
			}
		}else{
			$limit['unit'] = 1;
			$packages = $this->getOrderPackage($order, array('weight_unit' => 'g', 'volume_unit' => 'mm', 'limit' => $limit));

			if(empty($packages))
				return true;

			if(isset($packages['w']) || isset($packages['x']) || isset($packages['y']) || isset($packages['z'])){
				if(empty($parcels[$i]))
					$parcels[$i] = new stdClass();
				$parcels[$i]->Weight = $packages['w'];
				$parcels[$i]->Width = $packages['z'];
				$parcels[$i]->Height = $packages['y'];
				$parcels[$i]->Length = $packages['x'];

				if($parcels[$i]->Weight<1)$parcels[$i]->Weight=1;
				if($parcels[$i]->Length<150)$parcels[$i]->Length=150;
				if($parcels[$i]->Width<150)$parcels[$i]->Width=150;
				if($parcels[$i]->Height<1)$parcels[$i]->Height=1;

				$i++;
			}else{
				foreach($packages as $package){
					if(empty($parcels[$i]))
						$parcels[$i] = new stdClass();
					$parcels[$i]->Weight = $package['w'];
					$parcels[$i]->Width = $package['z'];
					$parcels[$i]->Height = $package['y'];
					$parcels[$i]->Length = $package['x'];

					if($parcels[$i]->Weight<1)$parcels[$i]->Weight=1;
					if($parcels[$i]->Length<150)$parcels[$i]->Length=150;
					if($parcels[$i]->Width<150)$parcels[$i]->Width=150;
					if($parcels[$i]->Height<1)$parcels[$i]->Height=1;

					$i++;
				}
			}

			foreach($parcels as $parcel){
				$parcel->Country = @$order->shipping_address_full->shipping_address->address_country->zone_code_2;
				if (empty($parcel->Country)) $parcel->Country='AU';
				$parcel->Pickup_Postcode = substr(trim(@$rate->shipping_params->post_code),0,4);
				$parcel->Destination_Postcode = substr(trim($order->shipping_address->address_post_code),0,4);
				$parcel->Quantity = 1;
				if (! empty($rate->shipping_params->PARCEL)){
					$this->addRate($rates,'PARCEL',$parcel, $rate, $currentCurrencyId, 1);
				}
				if (! empty($rate->shipping_params->SATCHEL)){
					$this->addRate($rates,'SATCHEL',$parcel, $rate, $currentCurrencyId, 1);
				}
			}
		}
	}
/**
 * Fastway On Shipping Configuration Save
 *
 * @version 	4th August 2015
 * @since	 	24th April 2015
 */
	function onShippingConfigurationSave(&$element) {


		if(empty($element -> shipping_params -> post_code)){
			$$this->queueError(JText::sprintf('ENTER_INFO', 'Fastway Pickup', JText::_('POST_CODE')));
		}

		if(empty($element -> shipping_params -> api_code)){
			$this->queueError(JText::sprintf('ENTER_INFO', 'Fastway', 'API Code. Fastway requires you to register for an API.  Please goto <a href="http://www.fastway.com.au/our-technology/api" title="Fastway Couriers API " target="_blank" >http://www.fastway.com.au/our-technology/api</a> to obtain an API Key.'));
		}

		$element->shipping_params->PARCEL=isset($element->shipping_params->services) && in_array('PARCEL',$element->shipping_params->services);
		$element->shipping_params->SATCHEL=isset($element->shipping_params->services) && in_array('SATCHEL',$element->shipping_params->services);
		parent::onShippingConfigurationSave($element);
	}
/**
 * Fastway Get Shipping Default Values
 *
 * @version 	4th August 2015
 * @since	 	24th April 2015
 */
	function getShippingDefaultValues(&$element){
		$element->shipping_name='Fastway Couriers';
		$element->shipping_description='';
		$element->shipping_images='aufastway';
		$element->shipping_params->PARCEL='PARCEL';
		$element->shipping_params->SATCHEL='SATCHEL';
		$element->shipping_params->post_code='';
		$element->shipping_params->api_code='';
		$element->shipping_params->pickup_locality='';
		$element->shipping_params->suppress_errors = false;
		$elements = array($element);
	}
/**
 * Fastway On Shipping Configuration
 *
 * @version 	4th August 2015
 * @since	 	24th April 2015
 */
	function onShippingConfiguration(&$element){
		$this->aufastway = JRequest::getCmd('name','aufastway');
		$this->categoryType = hikashop_get('type.categorysub');
		$this->categoryType->type = 'tax';
		$this->categoryType->field = 'category_id';

		parent::onShippingConfiguration($element);
	}
/**
 * Fastway Add Rate
 *
 * @version 	26th June 2015
 * @since	 	24th April 2015
 */
	function addRate(&$rates, $type ,$parcel, &$rate, $currency, $nb_package){

		$this->fastway = $this->resetFastwayObject();
		$this->fastway->pickup_locality = $rate->shipping_params->pickup_locality;
		$this->fastway->api_code = $rate->shipping_params->api_code;
		$this->fastway->post_code = $rate->shipping_params->post_code;
		$this->fastway->frequent_pickup = $rate->shipping_params->frequent_pickup;
		$fixed = $rate->shipping_params->limit_price;
		$free = (bool)$rate->shipping_params->free_shipping;
		$this->fastway->suppress_errors = $suppressErrors = (bool)$rate->shipping_params->suppress_errors;
		$this->fastway->lowest_only = $rate->shipping_params->lowest_only;
		$priceField = 'totalprice_normal';
		if ($rate->shipping_params->frequent_pickup) 
			$priceField = 'totalprice_frequent';
		if(empty($nb_package))
			$nb_package = 1;
		$rf = $this->retrievePickupFranchise($parcel);
		$parcel->Service_Type = $type;
		$this->fastway = $this->fastwayLookup($parcel);
		if(!empty($this->fastway->err_msg)){
			if($this->fastway->err_msg == 'OK'){
				if(empty($rates[$type])){
					$info = new stdClass();
					$info = (!HIKASHOP_PHP5) ? $rate : clone($rate);
					$info->shipping_name .= ' '.JText::_($type);
					$shipping_description = '';
					if ( JText::_($type.'_DESCRIPTION') != $type.'_DESCRIPTION')
						$shipping_description = JText::_($type.'_DESCRIPTION');
					if (!empty($rate->shipping_description))
						$info->shipping_description = trim($rate->shipping_description) . ' ' . $shipping_description;
					else{
						$info->shipping_description = $shipping_description;
					}
					$types = array('PARCEL' => 1, 'SATCHEL' => 2);
					$info->shipping_id .= '-' . $types[$type];
					$rates[$type]=$info;
				} else {
					$shipping_description = '';
					if ( JText::_(strtoupper($type).'_DESCRIPTION') != strtoupper($type).'_DESCRIPTION')
						$shipping_description = JText::_(strtoupper($type).'_DESCRIPTION');
					if(empty($shipping_description)){ $shipping_description = $rate->shipping_description; }
					if(!empty($shipping_description)){ $shipping_description .= '<br/>'; }
					if($nb_package > 1 && (isset($rate->shipping_params->shipping_group) && $rate->shipping_params->shipping_group)) $rates[$type]->shipping_description = $shipping_description . JText::sprintf('X_PACKAGES', $nb_package);
					else $rates[$type]->shipping_description = $shipping_description;
				}
				$x = NULL;
				$tot = NULL;
				$sp = 0;
				if (isset($rates[$type]->shipping_price)) {
					$sp = (float)$rates[$type]->shipping_price;
					if ($sp > 0 AND ! $fixed)
						$tot = $sp;
				}

				if (is_array($this->fastway->result->services))
					foreach($this->fastway->result->services as $q=>$w) {
						if (strtoupper($w->type) == strtoupper($type)) {
							if ($tot === NULL) {
								$tot = $w->$priceField;
								$x = $q;
							} elseif ($w->$priceField < $tot) {
								$tot = $w->$priceField;
								$x = $q;
							}
						}
					}
				if ($x !== NULL) {
					if(isset($rates[$type]->shipping_tax_id)){
						$currencyClass = hikashop_get('class.currency');
						if ($sp > 0 and $fixed) 
							$this->fastway->result->services[$x]->$priceField = $sp;
						if ($free)
							$this->fastway->result->services[$x]->$priceField = 0;
						$this->fastway->charge = $currencyClass->getUntaxedPrice($this->fastway->result->services[$x]->$priceField, hikashop_getZone(), $rates[$type]->shipping_tax_id);
					}
					$rates[$type]->shipping_price = $this->fastway->charge;
				} else {
					unset($rates[$type]);
					return true;
				}
			}elseif(!empty($this->fastway->err_msg) and ! $suppressErrors){
				$this->queueError('The request to the Fastway server failed with the message: '.$this->fastway->err_msg);
			}elseif (! $suppressErrors){
				$this->queueError('The request to the Fastway server failed');
			}
		}
	}
/**
 * Fastway Shipping Methods
 *
 * @version 	28th April 2015
 * @since	 	24th April 2015
 */
	function shippingMethods(&$main){
		$methods = array();
		if(!empty($main->shipping_params->services)){
			foreach($main->shipping_params->services as $service){
				$selected = null;
				foreach($this->methods as $name => $key){
					if($name == $service) {
						$selected = array('name' => $this->pluginConfig['services'][2][$name], 'key' => $key);
						break;
					}
				}
				if($selected){
					$methods[$main->shipping_id . '-' . $selected['key']] = ucfirst($selected['name']);
				}
			}
		}
		return $methods;
	}
/**
 * Fastway Pickup Franchise Retrieval
 *
 * @version 	19th June 2015
 * @since	 	27th April 2015
 * @return		string			Franchise RF
 */
	function retrievePickupFranchise($parcel){
		
		$url = $this->baseURL.'pickuprf/';
		$url .= $parcel->Pickup_Postcode.'/';
		switch ($parcel->Country) {
			default: $url .= '1/';
		}
		$locality = trim($this->fastway->pickup_locality);
		if (! empty($locality))
			$url .= str_replace(' ', '%20',$locality).'/';
		$result = $this->contactFastway($url);
		if (isset($result->result) and is_a($result->result, 'stdClass'))
			foreach((array)$result->result as $name=>$value)
				$this->fastway->$name = $value;
		else
			return false;
		return $this->fastway->franchise_code;
	}
/**
 * Fastway Shipping Methods
 *
 * @version 	19th June 2015
 * @since	 	27th April 2015
 * @param		string			URL
 * @return		object			Fastway Response
 */
	function contactFastway($url) {
		
		$url = rtrim($url, "/");
		if (! strpos($url, '?'))
			$url .= '?api_key='.$this->fastway->api_code;
		else
			$url .= '&api_key='.$this->fastway->api_code;
		$url = parse_url($url);
		if(!isset($url['port'])){
			if(!empty($url['scheme'])&&in_array($url['scheme'],array('https','ssl'))){
				$url['port'] = 443;
			} else {
				$url['port'] = 80;
			}
		}
		if(!empty($url['scheme'])&&in_array($url['scheme'],array('https','ssl'))){
			$url['host_socket'] = 'ssl://'.$url['host'];
		}else{
			$url['host_socket'] = $url['host'];
		}
		try {
			$fp = fsockopen ( $url['host_socket'], $url['port'], $errno, $errstr, 30);
		} catch (Exception $e) {
			$result = new stdClass();
			$this->queueError( JText::_('BLOCKED_ACCESS') );
			$result->err_msg = JText::_('BLOCKED_ACCESS') ;
			return $result;
		}
		if (!$fp) {
			$result = new stdClass();
			$this->queueError( JText::_('BLOCKED_ACCESS') );
			$result->err_msg = JText::_('BLOCKED_ACCESS') ;
			return $result;
		}
		$uri = $url['path'].($url['query']!='' ? '?' . $url['query'] : '');
		$header = "GET $uri HTTP/1.0\r\n".
			"User-Agent: PHP/".phpversion()."\r\n".
			"Referer: ".hikashop_currentURL()."\r\n".
			"Server: ".$_SERVER['SERVER_SOFTWARE']."\r\n".
			"Host: ".$url['host'].":".$url['port']."\r\n".
			"Accept: */"."*\r\n\r\n";

		fwrite($fp, $header);
		$response = '';
		while (!feof($fp)) {
			$response .= fgets ($fp, 1024);
		}
		fclose ($fp);
		$pos = strpos($response, "\r\n\r\n");
		$body = substr($response, ($pos - 4) + 2 * strlen("\r\n\r\n"));
		$result = json_decode($body, false);
		if (empty($result)) {
			$result = new stdClass;
			$result->err_msg = JText::_('BAD_FORMAT') ;
			$app = JFactory::getApplication();
			$app->enqueueMessage( JText::_('BAD_FORMAT') );
		} elseif (isset($result->error)) {
			$result->err_msg = $result->error;
			$this->queueError( $result->err_msg );
		}else
			$result->err_msg = 'OK';
		return $result ;
	}
/**
 * Fastway Object Reset
 *
 * @version 	1st September 2015
 * @since	 	27th April 2015
 * @return		stdClass			Fastway Object
 */
	function resetFastwayObject($force = false){
	
		if (! is_a($this->fastway, 'stdClass') or $force)
			$this->fastway = new stdClass;
		return $this->fastway;
	}
/**
 * Fastway Lookup
 *
 * Parameters:
 * RFCode - The RFCode of the pickup franchise (eg SYD, NPE, AUK). See the List Regional Franchises method for a more comprehensive list.
 * Suburb - The destination suburb (eg Homebush, Newcastle, Bunbury)
 * DestPostcode - The destination postcode (eg 2140, 4112, 4176)
 * WeightInKg - The total weight of the parcel, in kilograms. 25kg maximum for NZ/AU/IE, and 30kg maximum for SA
 * LengthInCm - Optional, used for specifying the length parameter of a cubic weight (in centimetres). If specified, Width and Height should also be specified.
 * WidthInCm - Optional, used for specifying the width parameter of a cubic weight (in centimetres). If specified, Length and Height should also be specified.
 * HeightInCm - Optional, used for specifying the height parameter of a cubic weight (in centimetres). If specified, Width and Length should also be specified.
 * AllowMultipleRegions - Optional, if set to 'true' then will return information about the multiple region matches instead of throwing an error
 * ShowBoxProduct - Optional, if set to 'true' it will also display box product options 
 * 
 * Example: http://api.fastway.org/v3/psc/lookup/AUK/Napier/4110?LengthInCm=30&WidthInCm=30&HeightInCm=30&api_key=YOUR_API_KEY
 * @version 	19th June 2015
 * @since	 	27th April 2015
 * @param		stdClass			Parcel Information
 * @return		stdClass			Fastway Object
 */
	function fastwayLookup($parcel){
	
		$url = $this->baseURL . 'lookup/';
		if (! isset($this->fastway->franchise_code)) {
			$result = new stdClass;
			$result->err_msg = ' A Fastway Pickup Franchise Code has not been found.';
			return $result ;
		}
		$url .= $this->fastway->franchise_code.'/';
		if (empty($this->fastway->address_city) and isset($this->fastway->result->to)) 
			$this->fastway->address_city = $this->fastway->result->to;
		if (empty($this->fastway->address_city)) {
			$result = new stdClass;
			$result->err_msg = 'No delivery location has been found.';
			return $result ;
		}
		$url .= str_replace(' ', '%20', $this->fastway->address_city).'/'.$parcel->Destination_Postcode.'/'.($parcel->Weight / 1000).'/';
		$url .= '?ShowBoxProduct=0&AllowMultipleRegions=1';
		$x = 10;  //(mm to cm)
		if ($parcel->Width * $parcel->Height * $parcel->Length > 0)
			$url .= '&LengthInCm=' . $parcel->Length / $x . '&WidthInCm=' . $parcel->Width / $x . '&HeightInCm=' . $parcel->Height / $x;
		else {
			$result = new stdClass ;
			$result->err_msg = JText::_('DIMENSIONS_ERROR');
			return $result;
		}
		$result = $this->contactFastway($url);
		if (! empty($result->error)) {
			$result->err_msg = $result->error;
			unset($result->error);
		}
		$x = array($url, $result);
		return $result;
	}
/**
 * Queue Error
 *
 * @version 	4th August 2015
 * @since	 	4th August 2015 
 * @param	string		text of Error Message.
 * @return 	void
 */
 	private function queueError($text)
	{
		
		if ($this->fastway->suppress_errors)
			return ;
		$app = JFactory::getApplication();
		$app->enqueueMessage($text);
		return ;
	}
}
